@@ -9,6 +9,7 @@ from collections import OrderedDict
 import matplotlib.pyplot as plt
 import h5py
 import reprlib
+from loguru import logger
 
 import pathml.core.masks
 
@@ -85,6 +86,9 @@ class Tile:
                 f"All label values must be of type str or np.ndarray or a number (i.e. a subdtype of np.number) "
             )
 
+        assert (
+            name != "None" and name != 0
+        ), "Cannot use values of '0' or 'None' as tile names"
         assert name is None or isinstance(
             name, str
         ), f"name is of type {type(name)} but must be of type str or None"
@@ -105,7 +109,7 @@ class Tile:
             # remove any Nones
             stain_type_dict = {key: val for key, val in stain_type_dict.items() if val}
             if stain_type_dict:
-                slide_type = pathml.core.types.SlideType(**stain_type_dict)
+                slide_type = pathml.core.slide_types.SlideType(**stain_type_dict)
 
         assert counts is None or isinstance(
             counts, anndata.AnnData
