@@ -5,6 +5,7 @@ License: GNU GPL 2.0
 
 import hashlib
 import os
+from loguru import logger
 from pathlib import Path
 
 import h5py
@@ -85,7 +86,7 @@ class DeepFocusDataModule(BaseDataModule):
 
     def _download_deepfocus(self, root):
         if self._check_integrity():
-            print("File already downloaded with correct hash.")
+            logger.info("File already downloaded with correct hash.")
             return
         self.data_dir.mkdir(parents=True, exist_ok=True)
         download_from_url(
